@@ -1,4 +1,4 @@
-globalVariables(c("value", "sigma", "condition", "x", "y", "name", "proflist", "delta", "loq", "bloq",
+utils::globalVariables(c("value", "sigma", "condition", "x", "y", "name", "proflist", "delta", "loq", "bloq",
                   "sigmaLS", "cbLower95", "cbUpper95", "cbLower68", "cbUpper68",
                   "iteration", "idx", "is.zero", "index", "converged", "iterations",
                   ".runbgOutput", "terminal", "parvalue", "dataErrorModel",
